'use strict';

var fs = require('fs');
var varint = require('varint');
var dagCbor = require('@ipld/dag-cbor');
var readerBrowser = require('../lib/reader-browser.js');
var indexer = require('../lib/indexer.js');
var CID = require('multiformats/cid');
var writer = require('../lib/writer.js');
var stream = require('stream');
var indexedReaderBrowser = require('../lib/indexed-reader-browser.js');
require('../car-browser.js');
var sha2 = require('multiformats/hashes/sha2');
var common = require('./common.js');
var garbage = require('garbage');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var fs__default = /*#__PURE__*/_interopDefaultLegacy(fs);
var varint__default = /*#__PURE__*/_interopDefaultLegacy(varint);
var CID__default = /*#__PURE__*/_interopDefaultLegacy(CID);
var garbage__default = /*#__PURE__*/_interopDefaultLegacy(garbage);

describe('Large CAR', () => {
  const objects = [];
  const cids = [];
  const expectedIndex = [];
  it('create, no roots', async () => {
    const {writer: writer$1, out} = writer['default'].create([]);
    stream.Readable.from(out).pipe(fs__default['default'].createWriteStream('./test.car'));
    let offset = dagCbor.encode({
      version: 1,
      roots: []
    }, 'dag-cbor').length;
    offset += varint__default['default'].encode(offset.length).length;
    for (let i = 0; i < 500; i++) {
      const obj = garbage__default['default'].object(1000);
      objects.push(obj);
      const bytes = dagCbor.encode(obj, 'dag-cbor');
      const hash = await sha2.sha256.digest(bytes);
      const cid = CID__default['default'].create(1, dagCbor.code, hash);
      cids.push(cid.toString());
      const blockLength = bytes.length;
      let length = cid.bytes.length + blockLength;
      const lengthLength = varint__default['default'].encode(length).length;
      length += lengthLength;
      const blockOffset = offset + lengthLength + cid.bytes.length;
      expectedIndex.push({
        cid,
        offset,
        length,
        blockOffset,
        blockLength
      });
      offset += length;
      await writer$1.put({
        cid,
        bytes
      });
    }
    await writer$1.close();
  });
  it('CarIndexer.fromIterable', async () => {
    const indexer$1 = await indexer.fromIterable(fs__default['default'].createReadStream('./test.car'));
    common.assert.deepStrictEqual(await indexer$1.getRoots(), []);
    let i = 0;
    for await (const blockIndex of indexer$1) {
      common.assert.deepStrictEqual(blockIndex, expectedIndex[i]);
      i++;
    }
  });
  it('CarIndexer.fromBytes', async () => {
    const indexer$1 = await indexer.fromBytes(await fs__default['default'].promises.readFile('./test.car'));
    common.assert.deepStrictEqual(await indexer$1.getRoots(), []);
    let i = 0;
    for await (const blockIndex of indexer$1) {
      common.assert.deepStrictEqual(blockIndex, expectedIndex[i]);
      i++;
    }
  });
  it('CarReader.fromBytes', async () => {
    const reader = await readerBrowser['default'].fromBytes(await fs__default['default'].promises.readFile('./test.car'));
    common.assert.deepStrictEqual(await reader.getRoots(), []);
    let i = 0;
    for await (const {cid, bytes} of reader.blocks()) {
      common.assert.strictEqual(cid.toString(), cids[i], `cid #${ i } ${ cid } <> ${ cids[i] }`);
      const obj = dagCbor.decode(bytes);
      common.assert.deepStrictEqual(obj, objects[i], `object #${ i }`);
      i++;
    }
  });
  it('CarReader.fromIterable', async () => {
    const reader = await readerBrowser['default'].fromIterable(fs__default['default'].createReadStream('./test.car'));
    common.assert.deepStrictEqual(await reader.getRoots(), []);
    let i = 0;
    for await (const {cid, bytes} of reader.blocks()) {
      common.assert.strictEqual(cid.toString(), cids[i], `cid #${ i } ${ cid } <> ${ cids[i] }`);
      const obj = dagCbor.decode(bytes);
      common.assert.deepStrictEqual(obj, objects[i], `object #${ i }`);
      i++;
    }
  });
  it('CarIndexedReader.fromFile', async () => {
    const reader = await indexedReaderBrowser['default'].fromFile('./test.car');
    common.assert.deepStrictEqual(await reader.getRoots(), []);
    let i = 0;
    for await (const {cid, bytes} of reader.blocks()) {
      common.assert.strictEqual(cid.toString(), cids[i], `cid #${ i } ${ cid } <> ${ cids[i] }`);
      const obj = dagCbor.decode(bytes);
      common.assert.deepStrictEqual(obj, objects[i], `object #${ i }`);
      i++;
    }
  });
  after(async () => {
    return fs__default['default'].promises.unlink('./test.car').catch(() => {
    });
  });
});
